# -*- coding: utf-8 -*-

from .comms import printer
from .parsers import VaspOutcar
from .parsers import AbinitOutput
from .parsers import QE_ElaStic_Parser
from .parsers import QE_thermo_pw_Parser
from .core import ElasticProperties
from .core import ElasticProperties2D
from .core import Structure
from numpy import array


def from_mp(mp_id, api_key, crystal=None, lattice_type=None, outfile=None):
    """
    This function retrieves the elastic tensor provided by the materials project database. 
    Please provide an API key generated from https://materialsproject.org/dashboard

    Parameters
    ----------
    mp_id : str
        material id in materials project database.
        e.g. ``mp_id = "mp-13"``
    api_key : str
        API key generated by materials project data base. https://materialsproject.org/dashboard
    crystal : TYPE, optional
        DESCRIPTION. The default is None.
    lattice_type : TYPE, optional
        DESCRIPTION. The default is None.
    outfile : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    None.

    """
    try:
        from pymatgen import MPRester
    except:
        print("To use this functionality please install pymatgen")

    # welcome message
    printer.print_mechelastic()
    crystal_type = crystal
    mpr = MPRester(api_key)
    data = mpr.query(
        criteria={"material_id": mp_id},
        properties=["formula", "elasticity", "structure"],
    )[0]
    elastic_tensor = array(data["elasticity"]["elastic_tensor_original"])
    mp_structure = data["structure"]
    symbols = [x for x in mp_structure.symbol_set]
    positions = mp_structure.frac_coords
    lattice = array(mp_structure.lattice.matrix)
    structure = Structure(symbols, positions, lattice)

    # elastic constants calculation for 3D materials
    elastic_properties = ElasticProperties(
        elastic_tensor, structure, crystal_type,)
    elastic_properties.print_properties()

    # other
    # else: We don't need this
    #     elastic_bulk.elastic_const_bulk(
    #         cnew, snew, crystal, cell, density, natoms, totalmass
    #     )

    print("\nThanks! See you later. ")
    return



# -*- coding: utf-8 -*-

import spglib
import numpy as np
from ..utils import elements
from ..utils.constants import N_avogadro


class Structure:
    def __init__(
        self, atoms, fractional_coordinates, lattice,
    ):
        self.fractional_coordinates = np.array(fractional_coordinates)
        self.atoms = np.array(atoms)
        self.lattice = np.array(lattice)

    @property
    def volume(self):
        return abs(np.linalg.det(self.lattice)) * 1e-30

    @property
    def masses(self):
        return [elements.atomic_mass(x) * 1.0e-3 for x in self.atoms]

    @property
    def density(self):
        return np.sum(self.masses) / (self.volume * N_avogadro)

    @property
    def species(self):
        return np.unique(self.atoms).tolist()

    @property
    def nspecies(self):
        return len(self.species)

    @property
    def natoms(self):
        return len(self.atoms)

    @property
    def atomic_numbers(self):
        return [elements.atomic_number(x) for x in self.atoms]

    @property
    def spglib_cell(self):
        return (self.lattice, self.fractional_coordinates, self.atomic_numbers)

    def get_space_group_number(self, symprec=1e-5):
        return spglib.get_symmetry_dataset(self.spglib_cell, symprec)["number"]

    def get_space_group_international(self, symprec=1e-5):
        return spglib.get_symmetry_dataset(self.spglib_cell, symprec)["international"]

    def get_wyckoff_positions(self, symprec=1e-5):
        return spglib.get_symmetry_dataset(self.spglib_cell, symprec)["wyckoffs"]
    
    def to_dict(self, symprec=1e-5):
        return {'atomic_numbers':self.atomic_numbers,
                'atoms':self.atoms.tolist(),
                'density':self.density,
                'fractional_coordinates':self.fractional_coordinates.tolist(),
                'space_group_international':self.get_space_group_international(symprec),
                'space_group_number':self.get_space_group_number(symprec),
                'wyckoff_positions':self.get_wyckoff_positions(symprec),
                'lattice':self.lattice.tolist(),
                'masses':self.masses,
                'natoms':self.natoms,
                'nspecies':self.nspecies,
                'species':self.species,
                'volume':self.volume,
            }


# -*- coding: utf-8 -*-
import numpy as np
import json
from dicttoxml import dicttoxml
from xml.dom.minidom import parseString
from ..comms import printer
from ..tests import ductile
from ..tests import eigenvals
from ..utils.constants import *
from ..utils.elements import ELEMENTS
from ..utils.crystalutils import *
from .structure import Structure


class ElasticProperties:
    def __init__(self, elastic_tensor, structure=None, crystal_type=None, verbose=True):
        """


        Parameters
        ----------
        elastic_tensor : TYPE
            DESCRIPTION.
        structure : TYPE, optional
            DESCRIPTION. The default is None.
        crystal_type : TYPE, optional
            DESCRIPTION. The default is None.

        Returns
        -------
        None.

        """
        self.elastic_tensor = np.matrix(elastic_tensor)
        self.compaliance_tensor = self.elastic_tensor.I
        self.structure = structure
        self.crystal_type = crystal_type
        self.verbose = verbose
        # if verbose:
        #     print_matrix(self.elastic_tensor) alredy printed

        eigenvals.positive_evals(self.elastic_tensor, verbose)

        if crystal_type is not None or structure is not None:

            crystal_select(
                cnew=self.elastic_tensor,
                cell=self.structure.spglib_cell,
                crystal_type=self.crystal_type,
                verbose=verbose,
            )
        if verbose:
            print(self.__str__)

    @property
    def K_v(self):
        """

        Returns
        -------
        KV : float
            Bulk Modulus Voigt.

        """
        cnew = self.elastic_tensor
        K_v = (cnew[0, 0] + cnew[1, 1] + cnew[2, 2]) + 2 * (
            cnew[0, 1] + cnew[1, 2] + cnew[2, 0]
        )
        K_v = K_v / 9.0
        return K_v

    @property
    def bulk_modulus_voigt(self):
        """


        Returns
        -------
        float
            Bulk Modulus Voigt.

        """
        return self.K_v

    @property
    def G_v(self):
        """


        Returns
        -------
        G_v : float
            Shear Modulus Voigt.

        """
        cnew = self.elastic_tensor
        ## Shear: Voigt
        G_v = (
            (cnew[0, 0] + cnew[1, 1] + cnew[2, 2])
            - (cnew[0, 1] + cnew[1, 2] + cnew[2, 0])
            + 3 * (cnew[3, 3] + cnew[4, 4] + cnew[5, 5])
        )
        G_v = G_v / 15.0
        return G_v

    @property
    def shear_modulus_voight(self):
        """


        Returns
        -------
        float
            Shear Modulus Voigt.

        """
        return self.G_v

    @property
    def E_v(self):
        """


        Returns
        -------
        float
            Young's Modulus.

        """
        return (9 * self.K_v * self.G_v) / (3 * self.K_v + self.G_v)

    @property
    def youngs_modulus_voigt(self):
        """


        Returns
        -------
        float
            Young's Modulus Voigt.

        """
        return self.E_v

    @property
    def Nu_v(self):
        """


        Returns
        -------
        float
            Poisson's Ratio Voigt.

        """
        return (3 * self.K_v - self.E_v) / (6 * self.K_v)

    @property
    def poissons_ratio_voigt(self):
        """


        Returns
        -------
        float
            Poisson's Ratio Voigt.

        """
        return self.Nu_v

    @property
    def M_v(self):
        """


        Returns
        -------
        float
            P-wave modulus voigt.

        """
        return self.K_v + (4 * self.G_v / 3.0)

    @property
    def p_wave_modulus_voigt(self):
        """


        Returns
        -------
        float
            P-wave modulus voigt.

        """
        return self.M_v

    @property
    def K_r(self):
        """


        Returns
        -------
        K_r : float
            Bulk Modulus Reuss.

        """
        snew = self.compaliance_tensor
        K_r = (snew[0, 0] + snew[1, 1] + snew[2, 2]) + 2 * (
            snew[0, 1] + snew[1, 2] + snew[2, 0]
        )
        K_r = 1.0 / K_r
        return K_r

    @property
    def bulk_modulus_reuss(self):
        """


        Returns
        -------
        float
            Bulk Modulus Reuss.

        """
        return self.K_r

    @property
    def G_r(self):
        """


        Returns
        -------
        G_r : float
            Shear Modulus Reuss.

        """
        snew = self.compaliance_tensor
        G_r = (
            4 * (snew[0, 0] + snew[1, 1] + snew[2, 2])
            - 4 * (snew[0, 1] + snew[1, 2] + snew[2, 0])
            + 3 * (snew[3, 3] + snew[4, 4] + snew[5, 5])
        )
        G_r = 15.0 / G_r
        return G_r

    @property
    def shear_modulus_reuss(self):
        """


        Returns
        -------
        G_r : float
            Shear Modulus Reuss.

        """
        return self.G_r

    @property
    def E_r(self):
        """


        Returns
        -------
        float
            Young's Modulus Reuss.

        """
        return (9 * self.K_r * self.G_r) / (3 * self.K_r + self.G_r)

    @property
    def youngs_modulus_reuss(self):
        """


        Returns
        -------
        float
            Young's modulus Reuss.

        """
        return self.E_r

    @property
    def Nu_r(self):
        """


        Returns
        -------
        float
            Poisson's Ration Reuss.

        """
        return (3 * self.K_r - self.E_r) / (6 * self.K_r)

    @property
    def poissons_ratio_reuss(self):
        """


        Returns
        -------
        float
            Poisson's Ration Reuss.

        """
        return self.Nu_r

    @property
    def M_r(self):
        """


        Returns
        -------
        float
            P-Wave Modulus Reuss.

        """
        return self.K_r + (4 * self.G_r / 3.0)

    @property
    def p_wave_modulus_reuss(self):
        """


        Returns
        -------
        float
            P-Wave Modulus Reuss.

        """
        return self.M_r

    @property
    def K_vrh(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Bulk Modulus.

        """
        return (self.K_r + self.K_v) / 2

    @property
    def bulk_modulus_voigt_reuss_hill(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Bulk Modulus.

        """
        return self.K_vrh

    @property
    def G_vrh(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Shear Modulus.

        """
        return (self.G_v + self.G_r) / 2

    @property
    def shear_modulus_voight_reuss_hill(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Shear Modulus.

        """
        return self.G_vrh

    @property
    def E_vrh(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Young's Modulus.

        """
        return (self.E_v + self.E_r) / 2

    @property
    def youngs_modulus_voigt_reuss_hill(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Young's Modulus.

        """
        return self.E_vrh

    @property
    def Nu_vrh(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Poisson's Modulus.

        """
        return (self.Nu_v + self.Nu_r) / 2.0

    @property
    def poissons_ratio_voigt_reuss_hill(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation Poisson's Modulus.

        """
        return self.Nu_vrh

    @property
    def M_vrh(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation P-wave Modulus.

        """
        return (self.M_v + self.M_r) / 2.0

    @property
    def p_wave_modulus_voigt_reuss_hill(self):
        """


        Returns
        -------
        float
            Voigt-Reuss-Hill Approximation P-wave Modulus.

        """
        return self.M_vrh

    @property
    def KG_ratio_v(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio voigt.

        """
        return self.K_v / self.G_v

    @property
    def KG_ratio_r(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio reuss.

        """
        return self.K_r / self.G_r

    @property
    def KG_ratio_vrh(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio voigt reuss hill.

        """
        return self.K_vrh / self.G_vrh

    @property
    def bulk_shear_ratio_voigt(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio voigt.

        """
        return self.K_v / self.G_v

    @property
    def bulk_shear_ratio_reuss(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio reuss.

        """
        return self.K_r / self.G_r

    @property
    def bulk_shear_ratio_voigt_reuss_hill(self):
        """


        Returns
        -------
        float
            Bulk/Shear ratio voigt reuss hill.

        """
        return self.K_vrh / self.G_vrh

    @property
    def A_z(self):
        """


        Returns
        -------
        float
            Zenner Anisotropy only for Cubic Crystals.

        """

        cnew = self.elastic_tensor
        return 2 * cnew[3, 3] / (cnew[0, 0] - cnew[0, 1])

    @property
    def anisotropy_zenner(self):
        """


        Returns
        -------
        float
            Zenner Anisotropy only for Cubic Crystals.

        """
        return self.A_z

    @property
    def A_cb(self):
        """


        Returns
        -------
        float
            Chung-Buessem only for Cubic Crystals.

        """

        a_cb = (self.G_v - self.G_r) / (self.G_v + self.G_r)
        return a_cb

    @property
    def anisotropy_Chung_Buessem(self):
        """


        Returns
        -------
        float
            Chung-Buessem  only for Cubic Crystals.

        """
        return self.A_cb

    @property
    def A_u(self):
        """


        Returns
        -------
        float
            Ranganathan and Ostoja-Starzewski method: Phys. Rev. Lett. 101, 055504 (2008).
            for any crystalline symmetry: Universal anisotropy index.
            Note: AU is a relative measure of anisotropy with respect to a limiting value.
            For example, AU does not prove that a crystal having AU = 3 has double the anisotropy
            of another crystal with AU = 1.5. I""'

        """
        return (self.K_v / self.K_r) + 5 * (self.G_v / self.G_r) - 6.0

    @property
    def anisotropy_universal(self):
        """


        Returns
        -------
        float
            Ranganathan and Ostoja-Starzewski method: Phys. Rev. Lett. 101, 055504 (2008).
            for any crystalline symmetry: Universal anisotropy index
            Note: AU is a relative measure of anisotropy with respect to a limiting value.
            For example, AU does not prove that a crystal having AU = 3 has double the anisotropy
            of another crystal with AU = 1.5. I""'

        """
        return self.A_u

    @property
    def A_l(self):
        """


        Returns
        -------
        float
               log-Euclidean anisotropy parameter by Christopher M. Kube, AIP Advances 6, 095209 (2016)
               AL  CV , CR   is based on the distance between the averaged stiffnesses CV and
               CR , which is more appropriate. Clearly, AL  CV , CR   is zero when the crystallite is isotropic.

        """
        return np.sqrt(
            (np.log(self.K_v / self.K_r)) ** 2 + 5 * (np.log(self.G_v / self.G_r)) ** 2
        )

    @property
    def anisotropy_log_euclidean(self):
        """


        Returns
        -------
        float
               log-Euclidean anisotropy parameter by Christopher M. Kube, AIP Advances 6, 095209 (2016)
               AL  CV , CR   is based on the distance between the averaged stiffnesses CV and
               CR , which is more appropriate. Clearly, AL  CV , CR   is zero when the crystallite is isotropic.

        """
        return self.A_l

    @property
    def ductility(self):
        """


        Returns
        -------
        string
            The ductility of the material

        """
        return ductile.ductile_test(self.KG_ratio_vrh)

    @property
    def anisotropy_log_euclidean(self):
        """


        Returns
        -------
        float
               log-Euclidean anisotropy parameter by Christopher M. Kube, AIP Advances 6, 095209 (2016)
               AL  CV , CR   is based on the distance between the averaged stiffnesses CV and
               CR , which is more appropriate. Clearly, AL  CV , CR   is zero when the crystallite is isotropic.

        """
        return self.A_l

    @property
    def velocity_transverse(self):
        """


        Returns
        -------
        float
            Transverse Sound Velocity(m/s) from Navier's equation using Voigt-Reuss-Hill Approximation

        """
        if self.structure is None:
            raise Exception("No structure was provided")

        G = self.G_vrh * 1.0e9
        return np.sqrt((G / self.structure.density))

    @property
    def velocity_logitudinal(self):
        """


        Returns
        -------
        float
            longitudinal Sound velocity(m/s) from Navier's equation using Voigt-Reuss-Hill Approximation

        """
        if self.structure is None:
            raise Exception("No structure was provided")
        G = self.G_vrh * 1.0e9  # converting from GPa to Pascal units (kg/ms^2)
        K = self.K_vrh * 1.0e9
        return np.sqrt(((3 * K + 4 * G) / (3.0 * self.structure.density)))

    @property
    def velocity_average(self):
        """


        Returns
        -------
        float
            Average Sound velocity(m/s)

        """
        vt = self.velocity_transverse
        vl = self.velocity_logitudinal

        return 1.0 / (
            np.cbrt((1.0 / 3.0) * (2.0 / (vt * vt * vt) + 1.0 / (vl * vl * vl)))
        )

    @property
    def debye_temperature(self):
        """


        Returns
        -------
        theta : float
            Debye temperature calculated using  Orson Anderson's proposal [Ref- J. Phys. Chem. Solids (1963) 24, 909-917].
            WARNING: Debye model for the atomic displacement is based on a monoatomic crystal, here we consider an average mass if your crystal has several species

        """
        if self.structure is None:
            raise Exception("No structure was provided")
        q = self.structure.natoms
        density = self.structure.density
        total_mass = np.sum(self.structure.masses)
        theta = (
            (h_Planck / kB)
            * self.velocity_average
            * np.cbrt((3 * q * N_avogadro * density) / (4 * (np.pi) * total_mass))
        )
        return theta

    @property
    def melting_temperature(self):
        """


        Returns
        -------
        float
                Melting temperature estimated using empirical relation from Ref: Johnston I, Keeler G, Rollins R and Spicklemire S 1996
                Solid State Physics Simulations, The Consortium for Upper-Level Physics Software (New York: Wiley)

        """
        return 607 + 9.3 * self.K_vrh

    @property
    def cauchy_pressure(self):
        """
        This parameter desceibes the nature of bonding
        CP > 0 (+ve) indicates that ionic bonding dominates
        CP < 0 (-ve) indicates that covalent bonding dominates
        Returns
        -------
        None.

        """
        return self.elastic_tensor[0, 1] - self.elastic_tensor[3, 3]

    @property
    def bonding_type(self):
        """
        This parameter desceibes the nature of bonding
        CP > 0 (+ve) indicates that ionic bonding dominates
        CP < 0 (-ve) indicates that covalent bonding dominates

        Returns
        -------
        str
            DESCRIPTION.

        """
        cauchy_pressure = self.cauchy_pressure
        if cauchy_pressure > 0:
            return "ionic"
        elif cauchy_pressure < 0:
            return "covalent"

    @property
    def kleinman_parameter(self):
        c = self.elastic_tensor
        return (c[0, 0] + 8 * c[0, 1]) / (7 * c[0, 0] - 2 * c[0, 1])

    @property
    def bond_bending_vs_streching(self):
        return

    @property
    def lambda_lame_coefficient(self):
        """


        Returns
        -------
        TYPE
            DESCRIPTION.

        """
        return self.E_vrh * self.Nu_vrh / ((1 + self.Nu_vrh) * (1 - 2 * self.Nu_vrh))

    @property
    def mu_lame_coefficient(self):
        return self.E_vrh / (2 * (1 + self.Nu_vrh))

    def to_dict(self, symprec=1e-5):
        """


        Parameters
        ----------
        symprec : float
            Precision used in calculating the space group in angstroms. The default is 1e-5.

        Returns
        -------
        dict
            DESCRIPTION.

        """
        return {
            "anisotropy_Chung_Buessem": self.anisotropy_Chung_Buessem,
            "anisotropy_log_euclidean": self.anisotropy_log_euclidean,
            "anisotropy_universal": self.anisotropy_universal,
            "anisotropy_zenner": self.anisotropy_zenner,
            "bond_bending_vs_streching": self.bond_bending_vs_streching,
            "bonding_type": self.bonding_type,
            "bulk_modulus_reuss": self.bulk_modulus_reuss,
            "bulk_modulus_voigt": self.bulk_modulus_voigt,
            "bulk_modulus_voigt_reuss_hill": self.bulk_modulus_voigt_reuss_hill,
            "bulk_shear_ratio_reuss": self.bulk_shear_ratio_reuss,
            "bulk_shear_ratio_voigt": self.bulk_shear_ratio_voigt,
            "bulk_shear_ratio_voigt_reuss_hill": self.bulk_shear_ratio_voigt_reuss_hill,
            "cauchy_pressure": self.cauchy_pressure,
            "compaliance_tensor": self.compaliance_tensor.tolist(),
            "crystal_type": self.crystal_type,
            "debye_temperature": self.debye_temperature,
            "ductility": self.ductility,
            "elastic_stability": self.elastic_stability,
            "elastic_tensor": self.elastic_tensor.tolist(),
            "hardness": self.hardness,
            "kleinman_parameter": self.kleinman_parameter,
            "lambda_lame_coefficient": self.lambda_lame_coefficient,
            "melting_temperature": self.melting_temperature,
            "mu_lame_coefficient": self.mu_lame_coefficient,
            "p_wave_modulus_reuss": self.p_wave_modulus_reuss,
            "p_wave_modulus_voigt": self.p_wave_modulus_voigt,
            "p_wave_modulus_voigt_reuss_hill": self.p_wave_modulus_voigt_reuss_hill,
            "poissons_ratio_reuss": self.poissons_ratio_reuss,
            "poissons_ratio_voigt": self.poissons_ratio_voigt,
            "poissons_ratio_voigt_reuss_hill": self.poissons_ratio_voigt_reuss_hill,
            "shear_modulus_reuss": self.shear_modulus_reuss,
            "shear_modulus_voight": self.shear_modulus_voight,
            "shear_modulus_voight_reuss_hill": self.shear_modulus_voight_reuss_hill,
            "structure": self.structure.to_dict(symprec),
            "velocity_average": self.velocity_average,
            "velocity_logitudinal": self.velocity_logitudinal,
            "velocity_transverse": self.velocity_transverse,
            "youngs_modulus_reuss": self.youngs_modulus_reuss,
            "youngs_modulus_voigt": self.youngs_modulus_voigt,
            "youngs_modulus_voigt_reuss_hill": self.youngs_modulus_voigt_reuss_hill,
        }

    def to_json(self, outfile="elastic_properties.json", symprec=1e-5):
        """


        Parameters
        ----------
        outfile : str, optional
            file path to the output file. The default is "elastic_properties.json".
        symprec : float, optional
            Precision used in calculating the space group in angstroms. The default is 1e-5.

        Returns
        -------
        None.

        """
        wf = open(outfile, "w")
        json.dump(
            self.to_dict(symprec), wf, sort_keys=True, indent=4, separators=(",", ": ")
        )
        wf.close()

    def to_xml(self, outfile="elastic_properties.xml", symprec=1e-5):
        """


        Parameters
        ----------
        outfile : str, optional
            file path to the output file. The default is "elastic_properties.xml".
        symprec : float, optional
            Precision used in calculating the space group in angstroms. The default is 1e-5.

        Returns
        -------
        None.

        """
        wf = open(outfile, "w")
        xml = dicttoxml(self.to_dict(symprec))
        dom = parseString(xml)
        wf.write(dom.toprettyxml())
        wf.close()

    def to_file(self, outfile="elastic_properties.txt"):
        """


        Parameters
        ----------
        outfile : str, optional
            Path to the output file. The default is "elastic_properties.txt".

        Returns
        -------
        None.

        """
        wf = open(outfile, "w")
        wf.write(self.__str__())
        wf.close()

    def __str__(self):
        """


        Returns
        -------
        None.

        """

        ret = ""
        ret += "\n------------------------------------------------------------------\n"
        ret += "Elastic Moduli\n"
        ret += "------------------------------------------------------------------\n\n"

        ret += "                          Voigt     Reuss    Average\n"
        ret += "-------------------------------------------------------\n"
        ret += "Bulk modulus   (GPa)  %9.3f %9.3f %9.3f \n" % (
            self.K_v,
            self.K_r,
            self.K_vrh,
        )
        ret += "Shear modulus  (GPa)  %9.3f %9.3f %9.3f \n" % (
            self.G_v,
            self.G_r,
            self.G_vrh,
        )
        ret += "Young's modulus  (GPa)  %9.3f %9.3f %9.3f \n" % (
            self.E_v,
            self.E_r,
            self.E_vrh,
        )
        ret += "Poisson's ratio         %9.3f %9.3f %9.3f \n" % (
            self.Nu_v,
            self.Nu_r,
            self.Nu_vrh,
        )
        ret += "P-wave modulus  (GPa) %9.3f %9.3f %9.3f \n" % (
            self.M_v,
            self.M_r,
            self.M_vrh,
        )
        ret += "Bulk/Shear ratio      %9.3f %9.3f %9.3f (%s) \n" % (
            self.KG_ratio_v,
            self.KG_ratio_r,
            self.KG_ratio_vrh,
            self.ductility,
        )

        ret += "\n------------------------------------------------------------------\n"
        ret += "Elastic parameters\n"
        ret += "------------------------------------------------------------------\n\n"

        ret += "Lame's first and second parameter; Ref.[3]\n"
        ret += "Lambda (GPa)  =  %10.3f  \n" % self.lambda_lame_coefficient
        ret += "Mu (GPa)  =  %10.3f  \n" % self.mu_lame_coefficient

        ret += "\n------------------------------------------------------------------\n"
        ret += "Bonding information\n"
        ret += "------------------------------------------------------------------\n \n"

        ret += "Kleinman’s parameter; Ref.[4,5]\n"
        ret += "NOTE: K = 0 (1) bending (stretching) would dominate\n"
        ret += "K =  %10.5f  \n" % self.kleinman_parameter
        ret += "\n"
        ret += "Cauchy's Pressure calculated from the relation : CP = C_12 - C_44\n"
        ret += "     CP > 0 (+ve) indicates that ionic bonding dominates\n"
        ret += "     CP < 0 (-ve) indicates that covalent bonding dominates\n"

        ret += "CP (GPa) =  %10.3f  \n" % self.cauchy_pressure
        ret += "Bonding is mainly " + self.bonding_type + "\n"

        ret += "\n------------------------------------------------------------------\n"
        ret += "Elastic Anisotropy\n"
        ret += "------------------------------------------------------------------\n\n"

        ret += (
            "Zener's anisotropy (true for cubic crystals only); Az = %6.3f; Ref.[6]\n"
            % self.A_z
        )
        ret += (
            "Chung-Buessem's anisotropy (true for cubic crystals only); Acb = %6.3f; Ref.[7]\n"
            % self.A_cb
        )
        ret += "Universal anisotropy index; Au = %6.3f; Ref.[8]\n" % self.A_u
        ret += "Log-Euclidean's anisotropy; AL = %6.3f; Ref.[9]\n" % self.A_l

        ret += "\n------------------------------------------------------------------\n"
        ret += "Elastic Wave Velocities and Debye Temperature\n"
        ret += "------------------------------------------------------------------\n\n"

        ret += (
            "Longitudinal wave velocity (vl) : %10.3f m/s; Ref.[10]\n"
            % self.velocity_logitudinal
        )
        ret += (
            "Transverse wave velocity (vt) : %10.3f m/s; Ref.[10]\n"
            % self.velocity_transverse
        )
        ret += (
            "Average wave velocity (vm) : %10.3f m/s; Ref.[10]\n"
            % self.velocity_average
        )
        ret += "Debye temperature  : %10.3f K; Ref.[10]\n" % self.debye_temperature
        ret += "\n"
        ret += "WARNING: The  Debye model for the atomic displacement is based on a monoatomic crystal approximation.\n"
        ret += "Here we consider an averaged mass, in case your crystal has several species.\n"

        ret += "\n------------------------------------------------------------------\n"
        ret += "Melting Temperature\n"
        ret += "------------------------------------------------------------------\n\n"

        ret += "Melting temperature calculated from the empirical relation: Tm = 607 + 9.3*Kvrh \pm 555 (in K); Ref.[11]"
        ret += "Tm =  %10.3f K (plus-minus 555 K) \n" % self.melting_temperature
        ret += "\n\n"
        ret += "WARNING: This is a crude approximation and its validity needs to be checked! \n"

        ret += "\n------------------------------------------------------------------\n"
        ret += "Hardness Analysis\n"
        ret += "------------------------------------------------------------------\n\n"

        H1a, H1b, H2, H3, H4, H5 = self.hardness
        ret += "Hardness (H1a) = {:.2f} GPa;  Ref.[12]\n".format(H1a)
        ret += "Hardness (H1b) = {:.2f} GPa;  Ref.[12]\n".format(H1b)
        ret += "Hardness (H2)  = {:.2f} GPa;  Ref.[13]\n".format(H2)
        ret += "Hardness (H3)  = {:.2f} GPa;  Ref.[14]\n".format(H3)
        ret += "Hardness (H4)  = {:.2f} GPa;  Ref.[15]\n".format(H4)
        ret += "Hardness (H5)  = {:.2f} GPa;  Ref.[16]\n".format(H5)
        ret += "\n"
        ret += """Hardness recommendation model:
********************************************************************
             Cubic  Hexagonal  Orthorhombic  Rhombohedral  General
********************************************************************
Insulator      H2      H1b          H2            H2          H2
Semiconductor  H5    H1b, H3                      H2          H5
Metal          H1a     H4           H4            H4          H4
********************************************************************
Insulator     : bandgap > 2 eV
Semiconductor : bandgap < 2 eV
Metal         : bandgap = 0 """

        ret += "\n------------------------------------------------------------------\n"
        ret += "References\n"
        ret += "------------------------------------------------------------------\n\n"
        ret += "[1] Necessary and Sufficient Elastic Stability Conditions in Various Crystal Systems. Félix Mouhat and François-Xavier Coudert. Phys. Rev. B (2014)\n"
        ret += "[2] Crystal Structures and Elastic Properties of Superhard IrN2 and IrN3 from First Principles. Zhi-jian Wu et al. Phys. Rev. B (2007)\n"
        ret += "[3]  The rock physics handbook, Cam-bridge university press. G. Mavko, T. Mukerji, J. Dvorkin. Cambridge University Press. (2020)\n"
        ret += "[4]  Deformation potentials in silicon. i. uniaxial strain. Leonard Kleinman. Phys.Rev. 128. (1962)\n"
        ret += "[5] Electronic Structure and the Properties of Solids: The Physics of the Chemical Bond. Walter A. Harrison.  (2012)\n"
        ret += "[6] Elasticity and Anelasticity of Metals. Clarence M. Zener et al. The Journal of Physical Chemistry. (1949)\n"
        ret += "[7] The Elastic Anisotropy of Crystals. D. H. Chung and W. R. Buessem. Journal of Applied Physics (1967)\n"
        ret += "[8] Universal Elastic Anisotropy Index. Shivakumar I. Ranganathan et al. Phys. Rev. Lett. (2008)\n"
        ret += "[9] Elastic Anisotropy of Crystals. Christopher M. Kube. AIP Advances. (2016)\n"
        ret += "[10] A Simplified Method for Calculating the Debye Temperature from Elastic Constants. Orson L.Anderson. Journal of Physics and Chemistry of Solids. (1963)\n"
        ret += "[11] Elastic constants versus melting temperature in metals. M.E.Fine et al. Scripta Metallurgica. (1984)\n"
        ret += "[12] Correlation between hardness and elastic moduli of the covalent crystals. Jiang, et al. (2011).\n"
        ret += "[13] Computational alchemy: the search for new superhard materials. Teter (1998).\n"
        ret += "[14] Mechanical and electronic properties of B12-based ternary crystals of orthorhombic phase. Jiang et al. (2010).\n"
        ret += "[15] Theoretical investigation on the transition-metal borides with Ta3B4-type structure: A class of hard and refractory materials. Miao et al. (2011).\n"
        ret += "[16] Modeling hardness of polycrystalline materials and bulk metallic glasses. Chen et al. (2011).\n"
        return ret

    @property
    def elastic_stability(self):
        return eigenvals.positive_evals(self.elastic_tensor, verbose=self.verbose)

    @property
    def hardness(self):
        """

        Returns
        -------
        float
            The hardness calculated by 6 different methods:
            [H1a and H1b] Correlation between hardness and elastic moduli of the covalent crystals. Jiang, et al. (2011).
            [H2] Computational alchemy: the search for new superhard materials. Teter (1998).
            [H3] Mechanical and electronic properties of B12-based ternary crystals of orthorhombic phase. Jiang et al. (2010).
            [H4] Theoretical investigation on the transition-metal borides with Ta3B4-type structure: A class of hard and refractory materials. Miao et al. (2011).
            [H5] Modeling hardness of polycrystalline materials and bulk metallic glasses. Chen et al. (2011).
        """
        B = self.K_vrh
        G = self.G_vrh
        Y = self.E_vrh
        v = self.Nu_vrh
        k = G / B
        H1a = (1 / 6.78) * G
        H1b = (1 / 16.48) * Y
        H2 = (0.1769 * G) - 2.899
        H3 = (1 / 15.76) * Y
        H4 = ((1 - 2 * v) * B) / (6 * (1 + v))
        H5 = 2 * ((k * k * G) ** 0.585) - 3
        return H1a, H1b, H2, H3, H4, H5


# def positive_evals(cnew, verbose=True):
#     """This method checks the postivity of the eigenvalues
#     of a matrix."""

#     # print("Eigen Values of the matrix:")
#     evals = list(np.linalg.eigvals(cnew))
#     evalsPrint = list(np.around(np.array(evals), 3))
#     if verbose:
#         print("%s" % evalsPrint)
#     check = 0
#     for i in range(len(evals)):
#         if evals[i] > 0.0:
#             pass
#         else:
#             check = 1
#     # if check == 1:
#     #     print(
#     #         "ATTENTION: One or more eigen values are negative indicating elastic instability."
#     #     )
#     # if check == 0:
#     #     print("All eigen values are positive indicating tic stability.")

#     return not (bool(check))


def print_matrix(c):
    print("\n------------------------------------------------------------------")
    print("Elastic Tensor (GPa units)")
    print("------------------------------------------------------------------\n")
    row = c.shape[0]
    col = c.shape[1]
    for i in range(row):
        for j in range(col):
            print("{:>10.3f} ".format(c[i, j]), end=" ")
            if j == (col - 1):
                print(" ")
    print("Note: For VASP users this is the pressure-corrected matrix")

    print("\n------------------------------------------------------------------")
    print("Elastic Tensor Eigen Values (GPa units)")
    print("------------------------------------------------------------------\n")

    return



#!/usr/bin/env python

"""
This file contains messages outputed to the screen.
"""

from ..version import version as __version__
from ..version import date as __date__


def print_mechelastic():
    # created at http://www.network-science.de/ascii/.
    mechelastic_str = """
      /\/\   ___  ___| |__   /__\ | __ _ ___| |_(_) ___
     /    \ / _ \/ __| '_ \ /_\ | |/ _` / __| __| |/ __|
    / /\/\ \  __/ (__| | | //__ | | (_| \__ \ |_| | (__
    \/    \/\___|\___|_| |_\__/ |_|\__,_|___/\__|_|\___| """
    print(("  \n" + mechelastic_str))
    print("\nA Python library to calculate elastic properties of materials.\n")
    print("Version %s created on %s\n" % (__version__, __date__))
    print("Please cite:")
    print(
        "- Sobhit Singh, Irais Valencia-Jaime, Olivia Pavlic, and Aldo H. Romero; Phys. Rev. B 97, 054108 (2018)."
    )
    print(
        "- Sobhit Singh, Logan Lang, Viviana Dovale-Farelo, Uthpala Herath, Pedram Tavadze, François-Xavier Coudert,\n Aldo H. Romero; arXiv:2012.04758 [cond-mat.mtrl-sci] (2020). "
    )
    print("\nDisclaimer:")
    print(
        "Please check the authenticity of your results before publishing. \nAUTHORS of this script do not guarantee the quality and/or accuracy of results generated using this script.\n"
    )
    print(
        "------------------------------------------------------------------------------------------"
    )

    return


def print_warning_2D():
    warning_str = """    The conversion of elastic constants from GPa to N/m units was done
              by multiplying the Cij matrix elements by the thickness of the simulation cell,
              i.e. by the out-of-plane 'c' lattice parameters. Of course, this assumes that
              vacuum is along the c-axis and this is a good approximation only for atomically
              thin monolayers. One needs to account for the thickness of the 2D system, if
              the system is considerably thick.
              I HOPE YOU UNDERSTAND WHAT YOU ARE DOING HERE. """
    print(("WARNING:  " + warning_str))

    return


def print_matrix(c):
    row = c.shape[0]
    col = c.shape[1]
    for i in range(row):
        for j in range(col):
            print("{:>10.4f} ".format(c[i, j]), end=" ")
            if j == (col - 1):
                print(" ")

    return



#!/usr/bin/env python

import numpy as np
from numpy import linalg as LA


def stability_test(matrix, crystal_type, verbose=True):
    """This methods tests for the stability of a structure, then returns a boolean if ther structure is stable.

        References
        [1] Necessary and Sufficient Elastic Stability Conditions in Various Crystal Systems. Félix Mouhat and François-Xavier Coudert. Phys. Rev. B (2014)
        [2] Crystal Structures and Elastic Properties of Superhard IrN2 and IrN3 from First Principles. Zhi-jian Wu et al. Phys. Rev. B (2007)
    """

    c = np.copy(matrix)
    stable = True
    to_print = ""
    if crystal_type == "cubic":
        to_print += ("Cubic crystal system \n")
        to_print += (
            "Born stability criteria for the stability of cubic system are: Ref.[1]  \n"
        )
        to_print += ("(i) C11 - C12 > 0;    (ii) C11 + 2C12 > 0;   (iii) C44 > 0 \n ")

        # check (i)   keep in mind list starts with 0, so c11 is stored as c00
        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if c[0][0] + 2 * c[0][1] > 0.0:
            to_print += ("Condition (ii) satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iii) satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) NOT satisfied.")
            condition3 = False

        conditions = (condition1, condition2, condition3)

        for condition in conditions:
            if condition == False:
                stable = False

        return stable

    if crystal_type == "hexagonal":
        to_print += ("Hexagonal crystal system \n")
        to_print += (
            "Born stability criteria for the stability of hexagonal system are: Ref.[1]  \n"
        )
        to_print += (
            "(i) C11 - C12 > 0;    (ii) 2*C13^2 < C33(C11 + C12);   (iii) C44 > 0 \n "
        )

        # check (i)   keep in mind list starts with 0, so c11 is stored as c00
        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if 2 * (c[0][2] * c[0][2]) < c[2][2] * (c[0][0] + c[0][1]):
            to_print += ("Condition (ii) satified.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iii) satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) NOT satisfied.")
            condition3 = False
        conditions = (condition1, condition2, condition3)

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "tetragonal":
        to_print += ("Tetragonal crystal system \n")
        to_print += (
            "Born stability criteria for the stability of Tetragonal system are: Ref.[1]  \n"
        )
        to_print += (
            "(i) C11 - C12 > 0;    (ii) 2*C13^2 < C33(C11 + C12);   (iii) C44 > 0; \n(iv) C66 > 0;    (v) 2C16^2 < C66*(C11-C12) \n "
        )

        # check (i)   keep in mind list starts with 0, so c11 is stored as c00
        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (i) is satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) is NOT satisfied.")
            condition1 = False

        if 2 * (c[0][2] * c[0][2]) < c[2][2] * (c[0][0] + c[0][1]):
            to_print += ("Condition (ii) is satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) is NOT satisfied.")
            condition2 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iii) is satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) is NOT satisfied.")
            condition3 = False

        if c[5][5] > 0.0:
            to_print += ("Condition (iv) is satisfied.")
            condition4 = True
        else:
            to_print += ("Condition (iv) is NOT satisfied.")
            condition4 = False

        if 2 * c[0][5] * c[0][5] < c[5][5] * (c[0][0] - c[0][1]):
            to_print += ("Condition (v) is satisfied.")
            condition5 = True
        else:
            to_print += ("Condition (v) is NOT satisfied.")
            condition5 = False

        conditions = (condition1, condition2,
                      condition3, condition4, condition5)

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "rhombohedral-1":
        to_print += ("Rhombohedral (class-1): point group: 3m, -3m and 32 \n")
        to_print += ("Born stability criteria for this class are: Ref.[1]  \n")
        to_print += (
            "(i) C11 - C12 > 0;    (ii) C13^2 < (1/2)*C33(C11 + C12);   (iii) C14^2 < (1/2)*C44*(C11-C12) = C44*C66; \n(iv)  C44 > 0; \n "
        )

        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (i) is satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) is NOT satisfied.")
            condition1 = False

        if (c[0][2] * c[0][2]) < (0.5) * c[2][2] * (c[0][0] + c[0][1]):
            to_print += ("Condition (ii) is satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) is NOT satisfied.")
            condition2 = False

        if c[0][3] * c[0][3] < 0.5 * c[3][3] * (c[0][0] - c[0][1]):
            to_print += ("Condition (iii) is satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) is NOT satisfied.")
            condition3 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iv) is satisfied.")
            condition4 = True
        else:
            to_print += ("Condition (iv) is NOT satisfied.")
            condition4 = False

        conditions = (condition1, condition2, condition3, condition4)

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "rhombohedral-2":
        to_print += ("Rhombohedral (class-2): i.e structures with point group: 3, -3 \n")
        to_print += (
            "Born stability criteria for the stability of Rhombohedral-1 class system are: Ref.[1]  \n"
        )
        to_print += (
            "(i) C11 - C12 > 0;    (ii) C13^2 < (1/2)*C33(C11 + C12);   (iii) C14^2 + C15^2 < (1/2)*C44*(C11-C12) = C44*C66; \n(iv)  C44 > 0;  Note: C15 is added.. \n "
        )

        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (i) is satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) is NOT satisfied.")
            condition1 = False

        if c[0][2] * c[0][2] < (0.5) * c[2][2] * (c[0][0] + c[0][1]):
            to_print += ("Condition (ii) is satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) is NOT satisfied.")
            condition2 = False

        if c[0][3] * c[0][3] + c[0][4] * c[0][4] < 0.5 * c[3][3] * (c[0][0] - c[0][1]):
            to_print += ("Condition (iii) is satified.")
            condition3 = True
        else:
            to_print += ("Condition (iii) is NOT satisfied.")
            condition3 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iv) is satisfied.")
            condition4 = True
        else:
            to_print += ("Condition (iv) is NOT satisfied.")
            condition4 = False

        conditions = (condition1, condition2, condition3, condition4)

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "orthorhombic":
        to_print += ("Orthorhombic crystal system.... \n")
        to_print += (
            "Born stability criteria for the stability of Orthorhombic systems are: Ref.[1]  \n"
        )
        to_print += (
            "(i) C11 > 0;   (ii) C11*C22 > C12^2;   (iii) C11*C22*C33 + 2C12*C13*C23 - C11*C23^2 - C22*C13^2 - C33*C12^2 > 0; \n(iv)  C44 > 0;   (v)  C55 > 0 ;   (vi)  C66 > 0 \n"
        )

        # check (i)   keep in mind list starts with 0, so c11 is stored as c00
        if c[0][0] > 0.0:
            to_print += ("Condition (i) is satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) is NOT satisfied.")
            condition1 = False

        if c[0][0] * c[1][1] > c[0][1] * (c[0][1]):
            to_print += ("Condition (ii) is satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) is NOT satisfied.")
            condition2 = False

        if (
            c[0][0] * c[1][1] * c[2][2]
            + 2 * c[0][1] * c[0][2] * c[1][2]
            - c[0][0] * c[1][2] * c[1][2]
            - c[1][1] * c[0][2] * c[0][2]
            - c[2][2] * c[0][1] * c[0][1]
            > 0
        ):
            to_print += ("Condition (iii) is satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) is NOT satisfied.")
            condition3 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iv) is satisfied.")
            condition4 = True
        else:
            to_print += ("Condition (iv) is NOT satisfied.")
            condition4 = False

        if c[4][4] > 0.0:
            to_print += ("Condition (v) is satisfied.")
            condition5 = True
        else:
            to_print += ("Condition (v) is NOT satisfied.")
            condition5 = False

        if c[5][5] > 0.0:
            to_print += ("Condition (vi) is satisfied.")
            condition6 = True
        else:
            to_print += ("Condition (vi) is NOT satisfied.")
            condition6 = False

        conditions = (
            condition1,
            condition2,
            condition3,
            condition4,
            condition5,
            condition6,
        )

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "monoclinic":
        to_print += ("Monoclinic crystal system.... \n")
        to_print += (
            "Born stability criteria for the stability of monoclinic systems are: Ref.[1,2]  \n"
        )
        to_print += (
            "(i) C11 > 0;  (ii)  C22 > 0; (iii)  C33 > 0; \n(iv)  C44 > 0;   (v)  C55 > 0 ;   (vi)  C66 > 0  "
        )
        to_print += (
            " (vii) [C11 + C22 + C33 + 2*(C12 + C13 + C23)] > 0;    (viii)  C33*C55 - C35^2 > 0; \n(ix)  C44*C66 - C46^2 > 0;   (x) C22 + C33 - 2*C23  > 0 "
        )
        to_print += (
            " (xi) C22*(C33*C55 - C35^2) + 2*C23*C25*C35 - (C23^2)*C55 - (C25^2)*C33   > 0  "
        )
        to_print += (
            " (xii)  2*[C15*C25*(C33*C12 - C13*C23) + C15*C35*(C22*C13 - C12*C23) + C25*C35*(C11*C23 - C12*C13)] - [C15*C15*(C22*C33 - C23^2) + C25*C25*(C11*C33 - C13^2) + C35*C35*(C11*C22 - C12^2)] + C55*g > 0  "
        )
        to_print += (
            "          where, g = [C11*C22*C33 - C11*C23*C23 - C22*C13*C13 - C33*C12*C12 + 2*C12*C13*C23 ] "
        )

        if c[0][0] > 0.0:
            to_print += ("Condition (i) is satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) is NOT satisfied.")
            condition1 = False

        if c[1][1] > 0.0:
            to_print += ("Condition (ii) is satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) is NOT satisfied.")
            condition2 = False

        if c[2][2] > 0.0:
            to_print += ("Condition (iii) is satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) is NOT satisfied.")
            condition3 = False

        if c[3][3] > 0.0:
            to_print += ("Condition (iv) is satisfied.")
            condition4 = True
        else:
            to_print += ("Condition (iv) is NOT satisfied.")
            condition4 = False

        if c[4][4] > 0.0:
            to_print += ("Condition (v) is satisfied.")
            condition5 = True
        else:
            to_print += ("Condition (v) is NOT satisfied.")
            condition5 = False

        if c[5][5] > 0.0:
            to_print += ("Condition (vi) is satisfied.")
            condition6 = True
        else:
            to_print += ("Condition (vi) is NOT satisfied.")
            condition6 = False

        # for i in range(0, 6):
        #    if(c[i][i]  > 0.0):
        #        to_print += ("Condition (%2d) is satified." % (i+1))
        #    else:
        #        to_print += ("Condition (%2d) is NOT satified." % (i+1))

        if c[0][0] + c[1][1] + c[2][2] + 2 * (c[0][1] + c[0][2] + c[1][2]) > 0:
            to_print += ("Condition (vii) is satisfied.")
            condition7 = True
        else:
            to_print += ("Condition (vii) is NOT satisfied.")
            condition7 = False

        if c[2][2] * c[4][4] - c[2][4] * c[2][4] > 0:
            to_print += ("Condition (viii) is satisfied.")
            condition8 = True
        else:
            to_print += ("Condition (viii) is NOT satisfied.")
            condition8 = False

        if c[3][3] * c[5][5] - c[3][5] * c[3][5] > 0.0:
            to_print += ("Condition (ix) is satisfied.")
            condition9 = True
        else:
            to_print += ("Condition (ix) is NOT satisfied.")
            condition9 = False

        if c[1][1] + c[2][2] - 2 * c[1][2] > 0.0:
            to_print += ("Condition (x) is satisfied.")
            condition10 = True
        else:
            to_print += ("Condition (x) is NOT satisfied.")
            condition10 = False

        if (
            c[1][1] * (c[2][2] * c[4][4] - c[2][4] * c[2][4])
            + 2 * c[1][2] * c[1][4] * c[2][4]
            - c[1][2] * c[1][2] * c[4][4]
            - c[1][4] * c[1][4] * c[2][2]
            > 0.0
        ):
            to_print += ("Condition (xi) is satisfied.")
            condition11 = True
        else:
            to_print += ("Condition (xi) is NOT satisfied.")
            condition11 = False

        g = (
            (c[0][0] * c[1][1] * c[2][2])
            - (c[0][0] * c[1][2] * c[1][2])
            - (c[1][1] * c[0][2] * c[0][2])
            - (c[2][2] * c[0][1] * c[0][1])
            + 2.0 * (c[0][1] * c[0][2] * c[1][2])
        )

        h1 = 2 * (
            c[0][4] * c[1][4] * (c[2][2] * c[0][1] - c[0][2] * c[1][2])
            + c[0][4] * c[2][4] * (c[1][1] * c[0][2] - c[0][1] * c[1][2])
            + c[1][4] * c[2][4] * (c[0][0] * c[1][2] - c[0][1] * c[0][2])
        )

        h2 = (
            c[0][4] * c[0][4] * (c[1][1] * c[2][2] - c[1][2] * c[1][2])
            + c[1][4] * c[1][4] * (c[0][0] * c[2][2] - c[0][2] * c[0][2])
            + c[2][4] * c[2][4] * (c[0][0] * c[1][1] - c[0][1] * c[0][1])
        )

        x = h1 - h2 + c[4][4] * g

        #  to_print +=  'x = %10.4f' % x
        #  to_print +=  'g = %10.4f' % g

        if x > 0.0:
            to_print += ("Condition (xii) is satisfied.")
            condition12 = True
        else:
            to_print += ("Condition (xii) is NOT satisfied.")
            condition12 = False
        conditions = (
            condition1,
            condition2,
            condition3,
            condition4,
            condition5,
            condition6,
            condition7,
            condition8,
            condition9,
            condition10,
            condition11,
            condition12,
        )

        for condition in conditions:
            if condition == False:
                stable = False

    if crystal_type == "triclinic":
        to_print += ("Triclinic crystal system.... \n")
        to_print += (
            "Triclinic systems only criteria for stability are positive eigen values of the elastic tensor  \n"
        )
        evals = list(LA.eigvals(c))
        evals_print = list(np.around(np.array(evals), 3))
        to_print += ("%s" % evals_print)
        check = 0
        for i in range(len(evals)):
            if evals[i] > 0.0:
                stable = True
                pass
            else:
                check = 1
                stable = False

    if verbose:
        print(to_print)
    return stable


def stability_test_2d(matrix, lattice_type, verbose):
    c = np.zeros(shape=(3, 3))

    c[0, 0] = matrix[0, 0]
    c[0, 1] = matrix[0, 1]
    c[0, 2] = matrix[0, 5]

    c[1, 0] = matrix[1, 0]
    c[1, 1] = matrix[1, 1]
    c[1, 2] = matrix[1, 5]

    c[2, 0] = matrix[5, 0]
    c[2, 1] = matrix[5, 1]
    c[2, 2] = matrix[5, 5]

    stable = True
    to_print = ""
    if lattice_type == "hexagonal":
        to_print += ("Hexagonal lattice \n")
        to_print += ("Stability criteria for the stability of hexagonal system are: \n")
        to_print += ("(i) C11 + C12 > 0;    (ii) C11 - C12 > 0;  \n ")

        if c[0][0] + c[0][1] > 0.0:
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (ii) satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        conditions = (condition1, condition2)

        for condition in conditions:
            if condition == False:
                stable = False

    elif lattice_type == "square":
        to_print += ("Square lattice \n")
        to_print += ("Stability criteria for the stability of square system are: \n")
        to_print += ("(i) C11 + C12 > 0;    (ii) C11 - C12 > 0;  (iii) C33 > 0     \n ")

        if c[0][0] + c[0][1] > 0.0:
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if c[0][0] - c[0][1] > 0.0:
            to_print += ("Condition (ii) satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        if c[2][2] > 0.0:
            to_print += ("Condition (iii) satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) NOT satisfied.")
            condition3 = False

        conditions = (condition1, condition2, condition3)

        for condition in conditions:
            if condition == False:
                stable = False

    elif lattice_type == "rectangular" or lattice_type == "rectangular-center":
        to_print += ("Rectangular lattice or Rectangular Center lattice \n")
        to_print += (
            "Stability criteria for the stability of rectangular lattice or rectangular Center lattic are: \n"
        )
        to_print += (
            "(i) 1/2*(C11 + C22 + (4(C12)**2 - (C11 -C22))**0.5)> 0;    (ii) 1/2*(C11 + C22 - (4(C12)**2 - (C11 -C22))**0.5)> 0 ;  (iii) C33 > 0     \n "
        )

        if (
            0.5
            * (
                c[0][0]
                + c[1][1]
                + (4 * (c[0][1]) ** 2 + (c[0][0] - c[1][1]) ** 2) ** 0.5
            )
            > 0.0
        ):
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if (
            0.5
            * (
                c[0][0]
                + c[1][1]
                - (4 * (c[0][1]) ** 2 + (c[0][0] - c[1][1]) ** 2) ** 0.5
            )
            > 0.0
        ):
            to_print += ("Condition (ii) satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        if c[2][5] > 0.0:
            to_print += ("Condition (iii) satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) NOT satisfied.")
            condition3 = False

        conditions = (condition1, condition2, condition3)

        for condition in conditions:
            if condition == False:
                stable = False

    elif lattice_type == "oblique":
        to_print += ("Oblique lattice \n")
        to_print += ("Stability criteria for the stability of oblique lattice are: \n")
        to_print += ("(i) C11 > 0 ;    (ii) C11 *C22 > (C12)*2 ;  (iii) det(Cij) > 0     \n ")

        if c[0][0] > 0.0:
            to_print += ("Condition (i) satisfied.")
            condition1 = True
        else:
            to_print += ("Condition (i) NOT satisfied.")
            condition1 = False

        if c[0][0] * c[1][1] > (c[0][1]) ** 2:
            to_print += ("Condition (ii) satisfied.")
            condition2 = True
        else:
            to_print += ("Condition (ii) NOT satisfied.")
            condition2 = False

        if np.linalg.det(c) > 0.0:
            to_print += ("Condition (iii) satisfied.")
            condition3 = True
        else:
            to_print += ("Condition (iii) NOT satisfied.")
            condition3 = False

        conditions = (condition1, condition2, condition3)

        for condition in conditions:
            if condition == False:
                stable = False
    else:
        raise NameError("lattice type %s not in recognized" % lattice_type)

    if verbose:
        print(to_print)
    return stable



from .surface import Surface
import numpy as np
from scipy import interpolate
from skimage import measure

__author__ = "Pedram Tavadze"
__maintainer__ = "Pedram Tavadze"
__email__ = "petavazohi@mail.wvu.edu"
__date__ = "March 31, 2020"


class Isosurface(Surface):
    def __init__(
            self,
            XYZ=None,
            V=None,
            isovalue=None,
            V_matrix=None,
            algorithm='lewiner',
            interpolation_factor=1,
            padding=None,
            transform_matrix=None,
            boundaries=None,
    ):
        """
        This class contains a surface that finds all the poins correcponding 
        to the following equation
        V(X,Y,Z) = f

        Parameters
        ----------
        XYZ : TYPE, list of lists of floats, (n,3)
            DESCRIPTION. a list of coordinates [[x1,y1,z1],[x2,y2,z2],...] 
            corresponding V
        V : TYPE, list of floats, (n,)
            DESCRIPTION. a list of values [V1,V2,...] corresponding to XYZ
            XYZ[0] >>> V[0]
            XYZ[1] >>> V[1]
        isovalue : TYPE, float
            DESCRIPTION. The constant value of the surface (f)
        V_matrix : TYPE, float (nx,ny,nz) 
            DESCRIPTION. one can present V_matrix instead of XYZ and V. 
            V_matrix is a matrix representation of XYZ and V together. This 
            matrix is generated if XYZ and V are provided. 
        algorithm : TYPE, string
            DESCRIPTION. The default is 'lewiner'. The algorithm used to find the isosurface, This 
            function used scikit-image to find the isosurface. possibilities 
            ['classic','lewiner']
        interpolation_factor : TYPE, int
            DESCRIPTION. The default is 1. This module uses Fourier Transform 
            interpolation. interpolation factor will increase the grid points
            in each direction by a this factor, the dafault is set to 1
        padding : TYPE, list of float (3,)
            DESCRIPTION. padding is used for periodic datasets such as bands in
            a solid state calculation. e.g. The 1st BZ is not covered fully so
            one might want to pad the matrix with wrap(look at padding in 
            numpy for wrap), afterwards one has to clip the surface to the 
            first BZ. easily doable using pyvista of trimesh
            padding goes as follows np.pad(self.eigen_matrix,
                              ((padding[0]/2, padding[0]/2),
                              (padding[1]/2, padding[1]/2)
                              (padding[2]/2, padding[2])),
                              "wrap")
            In other words it creates a super cell withpadding
        transform_matrix : TYPE, (3,3) float
            DESCRIPTION. applies an transformation to the vertices VERTS_prime=T*VERTS
        boundaries : TYPE, pyprocar surface
            DESCRIPTION. The default is None. The boundaries in which the isosurface will be clipped with
            for example the first brillouin zone 

        """

        self.XYZ = np.array(XYZ)
        self.V = V
        self.isovalue = isovalue
        self.V_matrix = V_matrix
        self.algorithm = algorithm
        self.padding = padding
        self.interpolation_factor = interpolation_factor
        self.transform_matrix = transform_matrix
        self.boundaries = boundaries
        if self.algorithm not in ['classic', 'lewiner']:
            print(
                "The algorithm chose has to be from ['classic','lewiner'], automtically choosing 'lewiner'"
            )
            self.algorithm = 'lewiner'

        if self.V_matrix is None:
            self.V_matrix = map2matrix(self.XYZ, self.V)

        if self.padding is None:
            self.padding = [self.nX // 2, self.nY // 2, self.nZ // 2]
        else:
            self.padding = [
                self.nX // 2 * padding[0], self.nY // 2 * padding[1],
                self.nZ // 2 * padding[2]
            ]
       

        verts, faces, normals, values = self._get_isosurface(
            interpolation_factor)

        if verts is not None and faces is not None:
            if transform_matrix is not None:
                verts = np.dot(verts, transform_matrix)
            """
            Python, unlike statically typed languages such as Java, allows complete
            freedom when calling methods during object initialization. However, 
            standard object-oriented principles apply to Python classes using deep 
            inheritance hierarchies. Therefore the developer has responsibility for 
            ensuring that objects are properly initialized when there are multiple 
            __init__ methods that need to be called.
            For this reason I will make one temporary surface and from there I will
            using the other surface provided.
            """
            if boundaries is not None:
                suprecell_surface = Surface(verts=verts,
                                            faces=faces,
                                            face_normals=normals)
                if not np.isnan(suprecell_surface.pyvista_obj.points[0,0]):
                    verts, faces = self.clip(suprecell_surface, boundaries)

        Surface.__init__(self, verts=verts, faces=faces, face_normals=normals)

    def clip(self, S1, S2):
        """
        This function clips S1 using the boundaries of S2 and returns 

        Parameters
        ----------
        S1 : TYPE pyprocar surface
            DESCRIPTION.
        S2 : TYPE pyprocar surface
            DESCRIPTION.

        Returns
        -------
        verts,faces

        """


        for iface in range(len(S2.faces)):
            normal = S2.face_normals[iface]
            
            
            center = np.average(S2.verts[S2.faces[iface]], axis=0)



            S1.pyvista_obj.clip(origin=center, normal=normal, inplace=True)
        

        faces = []
        courser = 0
        for i in range(S1.pyvista_obj.n_faces):
            npoints = S1.pyvista_obj.faces[courser]
            face = []
            courser += 1
            for ipoint in range(npoints):
                face.append(S1.pyvista_obj.faces[courser])
                courser += 1
            faces.append(face)
            

        return S1.pyvista_obj.points, faces

    @property
    def X(self):
        """
        

        Returns
        -------
        TYPE numpy array 
            DESCRIPTION. list of grids in X direction

        """
        return np.unique(self.XYZ[:, 0])

    @property
    def Y(self):
        """
        

        Returns
        -------
        TYPE numpy array 
            DESCRIPTION. list of grids in Y direction
        """
        return np.unique(self.XYZ[:, 1])

    @property
    def Z(self):
        """
        

        Returns
        -------
        TYPE numpy array 
            DESCRIPTION. list of grids in Z direction

        """
        return np.unique(self.XYZ[:, 2])

    @property
    def dxyz(self):
        """
        

        Returns
        -------
        list
            DESCRIPTION. length between points in each direction

        """
        dx = np.abs(self.X[-1] - self.X[-2])
        dy = np.abs(self.Y[-1] - self.Y[-2])
        dz = np.abs(self.Z[-1] - self.Z[-2])
        return [dx, dy, dz]

    @property
    def nX(self):
        """
        

        Returns
        -------
        TYPE int
            DESCRIPTION. number of points in the grid in X direction

        """
        return len(self.X)

    @property
    def nY(self):
        """
        

        Returns
        -------
        TYPE int
            DESCRIPTION. number of points in the grid in Y direction

        """
        return len(self.Y)

    @property
    def nZ(self):
        """
        

        Returns
        -------
        TYPE int
            DESCRIPTION. number of points in the grid in Z direction

        """
        return len(self.Z)

    @property
    def surface_boundaries(self):
        """
        This function tries to find the isosurface using no interpolation to find the 
        correct positions of the surface to be able to shift to the interpolated one
        to the correct position

        Returns
        -------
        list of tuples 
            DESCRIPTION. [(mins[0],maxs[0]),(mins[1],maxs[1]),(mins[2],maxs[2])]

        """

        padding_x = self.padding[0]
        padding_y = self.padding[1]
        padding_z = self.padding[2]

        eigen_matrix = np.pad(self.V_matrix,
                              ((padding_x, padding_x), (padding_y, padding_y),
                               (padding_z, padding_z)), "wrap")
        try:
            verts, faces, normals, values = measure.marching_cubes_lewiner(
                eigen_matrix, self.isovalue)
            for ix in range(3):
                verts[:, ix] -= verts[:, ix].min()
                verts[:, ix] -= (verts[:, ix].max() -
                                 verts[:, ix].min()) / 2  #+self.origin[ix]
                verts[:, ix] *= self.dxyz[ix]
            mins = verts.min(axis=0)
            maxs = verts.max(axis=0)

            return [(mins[0], maxs[0]), (mins[1], maxs[1]), (mins[2], maxs[2])]
        except:
            return None

    def _get_isosurface(self, interp_factor=1):
        """
        

        Parameters
        ----------
        interp_factor : TYPE, optional
            DESCRIPTION. The default is 1.

        Returns
        -------
        TYPE
            DESCRIPTION. verts
        TYPE
            DESCRIPTION. faces
        TYPE
            DESCRIPTION. normals
        TYPE
            DESCRIPTION. vlues

        """

        # Amount of kpoints needed to add on to fully sample 1st BZ

        padding_x = self.padding[0]
        padding_y = self.padding[1]
        padding_z = self.padding[2]

        eigen_matrix = np.pad(self.V_matrix,
                              ((padding_x, padding_x), (padding_y, padding_y),
                               (padding_z, padding_z)), "wrap")
   
        bnd = self.surface_boundaries
        
        if interp_factor != 1:
            # Fourier interpolate the mapped function E(x,y,z)

            eigen_matrix = fft_interpolate(eigen_matrix, interp_factor)

            # after the FFT we loose the center of the BZ, using numpy roll we
            # bring back the center of the BZ
            # eigen_matrix = np.roll(eigen_matrix,4  ,
            #     axis=[0, 1, 2])

        try:
            
            verts, faces, normals, values = measure.marching_cubes_lewiner(
                eigen_matrix, self.isovalue)
            
        except BaseException:
            print("No isosurface for this band")
            return None, None, None, None
        # recenter

        for ix in range(3):
            verts[:, ix] -= verts[:, ix].min()
            verts[:, ix] -= (verts[:, ix].max() -
                             verts[:, ix].min()) / 2  #+self.origin[ix]
            verts[:, ix] *= self.dxyz[ix] / interp_factor
            if bnd is not None and interp_factor != 1:
                verts[:, ix] -= (verts[:, ix].min() - bnd[ix][0])
            #     x_shift = verts[:,0].min() - bnd[0]
            # y_shift = verts[:,1].min() - bnd[1]
            # z_shift = verts[:,2].min() - bnd[2]

        # transfare from fraction to cartesian
        # verts = np.dot(verts, self.reciprocal_)
        # new_faces = np.zeros(shape=(len(faces), 4))
        # new_faces[:, 0] = 3
        # new_faces[:, 1:] = faces
        # faces = new_faces
        return verts, faces, normals, values


def map2matrix(XYZ, V):
    """
    mapps an Irregular grid to a regular grid

    Parameters
    ----------
    XYZ : TYPE
        DESCRIPTION.
    V : TYPE
        DESCRIPTION.

    Returns
    -------
    mapped_func : TYPE
        DESCRIPTION.

    """
    XYZ = XYZ
    V = V
    X = np.unique(XYZ[:, 0])
    Y = np.unique(XYZ[:, 1])
    Z = np.unique(XYZ[:, 2])
    mapped_func = np.zeros(shape=(len(X), len(Y), len(Z)))
    #kpoint_matrix = np.zeros(shape=(len(kx), len(ky), len(kz), 3)) This was added to check if the mesh grid is working
    for ix in range(len(X)):
        condition1 = XYZ[:, 0] == X[ix]
        for iy in range(len(Y)):
            condition2 = XYZ[:, 1] == Y[iy]
            for iz in range(len(Z)):
                condition3 = XYZ[:, 2] == Z[iz]
                tot_cond = np.all([condition1, condition2, condition3], axis=0)
                if len(V[tot_cond]) != 0:
                    mapped_func[ix, iy, iz] = V[tot_cond][0]
                    # kpoint_matrix[ikx, iky, ikz] = [
                    #     kx[ikx], ky[iky], kz[ikz]]
                else:
                    mapped_func[ix, iy, iz] = np.nan
                    # kpoint_matrix[ikx, iky, ikz] = [np.nan, np.nan, np.nan]
    return mapped_func


def fft_interpolate(function, interpolation_factor=2):
    """
    if I = interpolation_factor
    This function withh recieve f(x,y,z) with dimensions of (nx,ny,nz)
    and returns f(x,y,z) with dimensions of (nx*I,ny*I,nz*I)
    """

    eigen_fft = np.fft.fftn(function)
    shifted_fft = np.fft.fftshift(eigen_fft)
    nx, ny, nz = np.array(shifted_fft.shape)
    pad_x = nx * (interpolation_factor - 1) // 2
    pad_y = ny * (interpolation_factor - 1) // 2
    pad_z = nz * (interpolation_factor - 1) // 2
    new_matrix = np.pad(
        shifted_fft,
        ((pad_x, pad_x), (pad_y, pad_y), (pad_z, pad_z)),
        "constant",
        constant_values=0,
    )
    
    new_matrix = np.fft.ifftshift(new_matrix)
    interpolated = np.real(np.fft.ifftn(new_matrix)) * (interpolation_factor**
                                                        3)

    return interpolated



#!/usr/bin/env python

import numpy as np


def positive_evals(cnew, verbose=True):
    """This method checks the postivity of the eigenvalues
    of a matrix."""
    to_print = ""
    to_print += ("\nEigen Values of the matrix CNEW:\n")
    evals = np.linalg.eigvals(cnew)
    to_print += ("%s" % list(np.around(np.array(evals), 3)))+"\n"
    check = 0
    for i in range(len(evals)):
        if evals[i] > 0.0:
            pass
        else:
            check = 1
    if check == 1:
        to_print += (
            "ATTENTION: One or more eigen values are negative indicating elastic instability.\n"
        )
    if check == 0:
        to_print += ("All eigen values are positive indicating elastic stability.\n")
    if verbose:
        print(to_print)
    return not(bool(check))



import pyvista
import trimesh
import numpy as np
from shutil import which
from matplotlib import cm
from matplotlib import colors as mpcolors

__author__ = "Pedram Tavadze"
__maintainer__ = "Pedram Tavadze"
__email__ = "petavazohi@mail.wvu.edu"
__date__ = "March 31, 2020"


class Surface(object):
    """
        Surface is a class that holds information about a surface
        To create a surface the minimum requirements are verts and faces

        Parameters
        ----------
        verts : list of float (nverts,3)
            The list of verticies that create the surface.
        faces : list of integers (nfaces,3)
            The default is None. The list of connectivity between
            verts that create the surface.
        face_normals : list of float (nfaces,3)
            The list of normal vectors to each face.
        vert_normals : list of float (nverts,3)
            The list of normal vectors to each vertex.
        face_colors : list of tuples floats (nfaces,3)
            The list of colors of each face.
            **example**:``face_colors=[(1,0,0),(1,0.5,0),...,(1,0,0)]``
        vert_colors : list of tuples floats (nfaces,3)
            The list of colors of each vertex.
        vectors : list of floats (nfaces,3)
            The list of vectors one wants to attach to the
            surface(glyphs) Only useful in pyvista objects
        scalars : list of floats (nfaces,)
            The list of scalars for each face. This can represent
            the color using a color map
                


    """
    def __init__(self,
                 verts=None,
                 faces=None,
                 face_normals=None,
                 vert_normals=None,
                 face_colors=None,
                 vert_colors=None,
                 vectors=None,
                 scalars=None):

        self.verts = verts
        self.faces = faces
        self.face_normals = face_normals
        self.vert_normals = vert_normals
        self.face_colors = face_colors
        self.vert_colors = vert_colors
        self.vectors = vectors
        self.scalars = scalars
        self.test = None
        self.pyvista_obj = None
        self.trimesh_obj = None

        if self.verts is not None and self.faces is not None:
            self._create_pyvista()
            # self._create_trimesh()
            if self.face_normals is None:
                self.face_normals = self.pyvista_obj.face_normals
            # if self.vert_normals is None:
            #     self.vert_normals=self.pyvista_obj.point_normals

    # @property
    # def mesh(self):
    #     return self.trimesh_obj

    # @property
    # def polydata(self):
    #     return self.polydata

    @property
    def centers(self):
        """
        Centers of faces
        Returns
        -------
        centers : list of floats (n,3)
            A list of centers of faces.

        """

        if self.verts is not None:
            centers = np.zeros(shape=(len(self.faces), 3))
            for iface in range(self.nfaces):
                centers[iface, 0:3] = np.average(self.verts[self.faces[iface]],
                                                 axis=0)
        else:
            centers = None
        return centers

    @property
    def nfaces(self):
        """
        Number of faces
        Returns
        -------
        int
           Number of faces in the surface.

        """
        return len(self.faces)

    @property
    def nverts(self):
        """
        Number or vertices.
        Returns
        -------
        int
            Number of verticies in in the surface.

        """
        return len(self.verts)

    @property
    def center_of_mass(self):
        """
        Center of mass of the vertices.
        Returns
        -------
        list float
            Center of mass of vertices.
        """
        return np.average(self.verts, axis=1)

    def _create_pyvista(self):
        """
        creates pyvista object this object has to have a certain order
        for faces
        example :
        [n_verts_1st_face,1st_vert,2nd_vert,...,nverts_2nd_face,1st_vert,2nd_vert,...]
        """
        verts = np.array(self.verts)
        faces = np.array(self.faces)
        new_faces = []

        for iface in faces:
            new_faces.append(len(iface))
            for ivert in iface:
                new_faces.append(ivert)
                
        self.pyvista_obj = pyvista.PolyData(verts, np.array(new_faces))
        if self.scalars is not None:
            self.pyvista_obj['scalars'] = self.scalars
            self.pyvista_obj.set_active_scalars('scalars')
        if self.vectors is not None:
            self.pyvista_obj['vectors'] = self.vectors
            self.pyvista_obj.set_active_vetors('vectors')

    def _create_trimesh(self):
        """
        creates a trimesh object
        """

        if np.any(np.array([len(x) for x in self.faces]) > 3):
            faces = []
            for i in range(0, len(self.pyvista_obj.triangulate().faces), 4):
                point_1 = self.pyvista_obj.triangulate().faces[i + 1]
                point_2 = self.pyvista_obj.triangulate().faces[i + 2]
                point_3 = self.pyvista_obj.triangulate().faces[i + 3]
                faces.append([point_1, point_2, point_3])
            self.trimesh_obj = trimesh.Trimesh(vertices=self.verts,
                                                faces=faces)

        else:

            self.trimesh_obj = trimesh.Trimesh(vertices=self.verts,
                                                faces=self.faces)

    def set_scalars(
            self,
            scalars,
    ):
        """
        Sets/Updates the scalars of the surface. Scalars represent a
        color using a color map.

        Parameters
        ----------
        scalars : list
            Scalars should be the same size as the number of faces.


        """
        self.scalars = scalars
        self.pyvista_obj['scalars'] = self.scalars
        self.pyvista_obj.set_active_scalars('scalars')

    def set_vectors(self, vectors_X, vectors_Y, vectors_Z):

        self.vectors = np.vstack([vectors_X, vectors_Y, vectors_Z]).T
        self.pyvista_obj['vectors'] = self.vectors
        # self.pyvista_obj.set_active_scalars('vectors')

        # self.pyvista_obj.vectors = self.vectors

    def set_color_with_cmap(self, cmap='viridis', vmin=None, vmax=None):
        """
        Sets colors for the trimesh object using the color map provided

        Parameters
        ----------
        cmap : TYPE, string
            DESCRIPTION. The default is 'viridis'.
        vmin : TYPE, float
            DESCRIPTION. The default is None.
        vmax : TYPE, optional
            DESCRIPTION. The default is None.


        """
        if vmin is None:
            vmin = min(self.scalars)
        if vmax is None:
            vmax = max(self.scalars)
        norm = mpcolors.Normalize(vmin=vmin, vmax=vmax)
        cmap = cm.get_cmap(cmap)

        colors = np.array([cmap(norm(x)) for x in self.scalars]).reshape(-1, 4)
        self.face_colors = colors

        # This next line will make all the surfaces double sided if you want
        # only show one side comment the next line
        if len(self.trimesh_obj.faces) == self.nfaces:
            self.trimesh_obj.faces = np.vstack(
                (self.trimesh_obj.faces, np.fliplr(self.trimesh_obj.faces)))

        if len(self.trimesh_obj.faces) == self.nfaces:
            self.trimesh_obj.visual.face_colors = colors
        else:
            self.trimesh_obj.visual.face_colors = np.append(colors,
                                                            colors,
                                                            axis=0)

    def export(self, file_obj='output.glb', file_type='glb'):
        """
        This function uses the export function from trimesh

        Parameters
        ----------
        file_obj : TYPE, optional
            DESCRIPTION. The default is 'output.glb'.
        file_type : TYPE, optional
            DESCRIPTION. The default is 'glb'.

        Returns
        -------
        None.

        """
        self.trimesh_obj.export(file_obj, file_type)


def convert_from_pyvista_faces(pyvista_obj):
    """
    pyvista mesh faces are written in a 1d array, This function returns faces in 
    a conventional way. A list of lists, where each list contains integers numbers of 
    vert conections

    Parameters
    ----------
    pyvista_obj : TYPE PyVista mesh
        DESCRIPTION.

    Returns
    -------
    new_faces : TYPE list of lists
        DESCRIPTION. A list of lists, where each list contains integers numbers of 
    vert conections

    """
    new_faces = []
    courser = 0
    for iface in range(pyvista_obj.n_faces):
        start = courser + 1
        end = start + pyvista_obj.faces[courser]
        face = pyvista_obj.faces[start:end]
        courser = end
        new_faces.append(face)
    return new_faces


def boolean_add(surfaces):
    """
    This functtion uses boolean add from PyVista 

    Parameters
    ----------
    surfaces : TYPE list of pyprocar.Surface
        DESCRIPTION. 

    Returns
    -------
    surf : TYPE  pyprocar surface
        DESCRIPTION. The unionized surface from surfaces

    """

    ret = surfaces[0].pyvista_obj.copy()
    for isurface in range(1, len(surfaces)):
        ret = ret.boolean_add(surfaces[isurface].pyvista_obj, inplace=False)
    surf = Surface(verts=ret.points,
                   faces=convert_from_pyvista_faces(ret),
                   face_normals=ret.face_normals,
                   vert_normals=ret.point_normals,
                   scalars=ret.active_scalars)
    return surf



from .comms import printer
from .parsers import VaspOutcar
from .parsers import AbinitOutput
from .parsers import QE_ElaStic_Parser
from .parsers import QE_thermo_pw_Parser

from .core import ELATE


def calculate_elastic_anisotropy(
    infile="OUTCAR",
    code="vasp",
    plot=None,
    elastic_calc=None,
    anaddbfile=None,
    outfile=None,
    adjust_pressure=True,
    npoints=100,
    show=True
):
    """
    This method calculates the elastic properties
    of a material from a DFT calculation. hi
    """

    # welcome message
    printer.print_mechelastic()

    elastic_tensor = None

    rowsList = []
    # calling parser
    if code == "vasp":

        output = VaspOutcar(infile=infile, adjust_pressure=adjust_pressure)
        elastic_tensor = output.elastic_tensor
        elastic_tensor = output.elastic_tensor
        density = output.density

        row = elastic_tensor.shape[0]
        col = elastic_tensor.shape[1]
        rowsList = []
        for i in range(row):
            columnsList = []
            for j in range(col):
                columnsList.append(round(elastic_tensor[i, j], 3))
            rowsList.append(columnsList)

    elif code == "abinit":
        output = AbinitOutput(infile=infile, anaddbfile=anaddbfile)
        elastic_tensor = output.elastic_tensor
        density = output.density

        row = elastic_tensor.shape[0]
        col = elastic_tensor.shape[1]
        rowsList = []
        for i in range(row):
            columnsList = []
            for j in range(col):
                columnsList.append(round(elastic_tensor[i, j], 3))
            rowsList.append(columnsList)

    elif code == "qe_ElaStic":
        output = QE_ElaStic_Parser(outfile=outfile, infile=infile)
        elastic_tensor = output.elastic_tensor
        density = output.density

        row = elastic_tensor.shape[0]
        col = elastic_tensor.shape[1]
        rowsList = []
        for i in range(row):
            columnsList = []
            for j in range(col):
                columnsList.append(round(elastic_tensor[i, j], 3))
            rowsList.append(columnsList)

    elif code == "qe_thermo_pw":
        output = QE_thermo_pw_Parser(outfile=outfile, infile=infile)
        elastic_tensor = output.elastic_tensor
        density = output.density

        row = elastic_tensor.shape[0]
        col = elastic_tensor.shape[1]
        rowsList = []
        for i in range(row):
            columnsList = []
            for j in range(col):
                columnsList.append(round(elastic_tensor[i, j], 3))
            rowsList.append(columnsList)

    print(rowsList)
    elastic_tensor = ELATE(rowsList, density)

    if plot == "2D":
        fig = elastic_tensor.plot_2D(
            elastic_calc=elastic_calc, npoints=npoints, apply_to_plot=None, show=show)
    elif plot == "3D":
        meshes = elastic_tensor.plot_3D(
            elastic_calc=elastic_calc, npoints=npoints, show=show)

    elastic_tensor.print_properties()

    print("\nThanks! See you later. ")
    if plot == "2D":
        return output, fig
    elif plot == "3D":
        return output, meshes
    else:
        return output



# Copyright (C) 2021 Sobhit Singh, Romero Group
#
# This file is part of MechElastic.
#
# MechElastic is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# MechElastic is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with MechElastic.  If not, see <http://www.gnu.org/licenses/>.

from .version import version as __version__
from .version import author as __author__
from .version import copyright as __copyright__
from .version import email as __email__
from .version import status as __status__
from .version import date as __date__


import re
import sys
import matplotlib.pyplot as plt
import numpy as np
import pyvista
import spglib
import networkx
import dicttoxml
import intersect
import pathlib

from mechelastic.parsers import VaspOutcar
from mechelastic.parsers import AbinitOutput
from mechelastic.eos import EOS
from mechelastic.comms import printer
from mechelastic.core import (
    Structure,
    ElasticProperties,
    ElasticProperties2D,
    ELATE,
    Surface,
    Isosurface,
    BrillouinZone,
)
from mechelastic.tests import ductile, eigenvals, stability, symmetry
from mechelastic.utils import constants, elements, crystalutils

from .calculate_elastic_anisotropy import calculate_elastic_anisotropy
from .calculate_elastic import calculate_elastic
from .from_mp import from_mp



#!/usr/bin/env python

"""This script reads elastic constants from DFT output files, calculates elastic moduli, and performs mechanical stability test:
   Authors: Sobhit Singh (1,2) Logan Lang (2), Viviana Dovale-Farelo (2), Uthpala Herath(2), Pedram Tavadze(2), François-Xavier Coudert(3), Aldo Romero (2)
   Email: smsingh@mix.wvu.edu, alromero@mail.wvu.edu
   (1) Rutgers University, Piscataway, NJ, USA
   (2) West Virginia University, Morgantown, WV, USA
   (3) Chimie ParisTech, PSL University, CNRS, Institut de Recherche de Chimie Paris, 75005, Paris, France
Version: 12.2019   #Dec. 2019

Please cite the below paper if you use this code for your research:
    - Sobhit Singh, Irais Valencia-Jaime, Olivia Pavlic, and Aldo H. Romero; Phys. Rev. B 97, 054108 (2018).
    - Sobhit Singh, Logan Lang, Viviana Dovale-Farelo, Uthpala Herath, Pedram Tavadze, François-Xavier Coudert, Aldo H. Romero; arXiv:2012.04758 [cond-mat.mtrl-sci].


To RUN this code for a 3D system assuming you know the crystal type before hand:
 >> python MechElastic.py -i OUTCAR-file -c hexagonal --dim 3D

Note: The crystal type is needed only to perform the elastic stabilty test.
      This test is not yet implemented for 2D systems, so you can ignore the crystal type for 2D systems.

If the crystal symmetry is not provided by user, then the code will rely on spglib to find it
 >> python MechElastic.py -i OUTCAR-file --dim 3D

To RUN this code for a 2D monolayer system
 >> python MechElastic.py -i OUTCAR-file --dim 2D
      (please pay attention to the warning for 2D systems)

OUTCAR file, if present in the current directory, is read as default unless a filename is specified by the user.


Disclaimer: Please check the authenticity of your results before publishing.
            AUTHORS of this script do not guarantee the quality and/or accuracy
            of results generated using this script.

"""

import argparse
import sys
import mechelastic

parser = argparse.ArgumentParser()
parser.add_argument(
    "-i",
    "--input",
    type=str,
    help="Output file of the DFT calculation.",
    default=None,
)
parser.add_argument(
    "-anaddb",
    "--anaddbfile",
    type=str,
    help="DDB file of the Abinit anaddb calculation.",
    default=None,
)
parser.add_argument(
    "-qe_outfile",
    type=str,
    help="Quantum Espresso output file.",
    default=None,
)

parser.add_argument(
    "-c",
    "--crystal",
    type=str,
    default=None,
    help="Provide the crystal family. Otherwise it would be determined from the DFT output.",
)
parser.add_argument(
    "-d",
    "--dim",
    type=str,
    help="Enter the dimension, 2D or 3D: For example: '-d 2D' or '--dim 2D'. Default is '3D' ",
    default="3D",
)
parser.add_argument(
    "-co",
    "--code",
    type=str,
    help="DFT code",
    default="vasp",
    choices=["vasp", "abinit", "qe"],
)
parser.add_argument(
    "-ap",
    "--adjust_pressure",
    default=1,
    type=int,
    help="Flag to adjust pressure in Elastic Tensor (VASP). Default: 1 (True)",
    choices=[1, 0],
)

parser.add_argument(
    "-v",
    "--verbose",
    action="store_true",
    help="Enable verbosity.",
)
args = parser.parse_args()

print("-----------------------------")
print("List of arguments entered:")
print("Input file name:", args.input)
print("DFT code:", args.code)
print("Crystal type: ", args.crystal)
print("Dimensions:", args.dim)
print("-----------------------------")


# calculate elastic properties
def main():

    if args.adjust_pressure == 1:
        args.adjust_pressure = True
    else:
        args.adjust_pressure = False

    mechelastic.calculate_elastic(
        code=args.code,
        dim=args.dim,
        infile=args.input,
        crystal=args.crystal,
        anaddbfile=args.anaddbfile,
        adjust_pressure=args.adjust_pressure,
        qe_outfile=args.qe_outfile,
        verbose=args.verbose,
    )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import numpy as np
from ..comms import printer
from ..tests import ductile
from ..tests import eigenvals
from ..utils.constants import *
from ..utils.elements import ELEMENTS
from ..utils.crystalutils import *
from .structure import Structure
from ..tests import stability

class ElasticProperties2D:
    """Convert the units from GPa or N/m2 to N/m for two-dimensional systems
        1 GPa = 10^9 N/m2

        Here, we use second Piola-Kirchhoff stress method to express the 2D forces per unit length in N/m units.
        Ref: [Peng et al., Acta Mechanica 223 (2012), 2591-2596; Comput. Mater. Sci. 68, 320 (2013);  Mech. Mater. 64, 135 (2013). ]
        [Singh et al., Phys. Rev. B 95, 165444 (2017)]

        We multiply elastic tensor by the thickness of the simulation cell to consider the vacuum thickness.
        In 2D:  Cij = bulk_Cij * C_latticevector (final units N/m)

        For example: if bulk_Cij = 15 GPa and out-of-plane cell parameter c = 10 Angs.
                  Then  2D_Cij = [15*10^9 N/m2] * [10*10^(-10) m] ; i.e 15*(0.1*c) N/m """

    def __init__(self, elastic_tensor, lattice_constant, structure = None , lattice_type = None):
        self.elastic_tensor = elastic_tensor
        self.lattice_constant = lattice_constant
        self.c2d = np.zeros((6, 6))
        
        self.structure = structure 
        self.lattice_type = lattice_type
        self._c2d()
        
        # if self.structure is not None: #or self.lattice_type is not None:
        lattice_select(
            cnew=self.c2d,
            cell=None,
            lattice_type=self.lattice_type,
        )

    def _c2d(self):
        for i in range(0, 6):
            for j in range(0, 6):
                self.c2d[i, j] = (
                    self.elastic_tensor[i, j] * 0.1 * float(self.lattice_constant[2])
                )

        print("\n \n Elastic tensor for two-dimensional system in N/m units \n")
        np.set_printoptions(precision=3, suppress=True)
        printer.printMatrix(self.c2d)
        
        # if self.lattice_type is not None or self.structure is not None:
        #     print(
        #         "\n------------------------------------------------------------------"
        #     )
        #     print("Mechanical Stability Tests")
        #     print(
        #         "------------------------------------------------------------------\n")
        #     self.elastic_stability

    # Elastic Properties

    @property
    def Lm(self):
        """
        Layer modulus: represents the resistance of a 2D sheet to stretching; Lm = (1/4)*[c11 + c22 + 2*c12]  [Ref: Andrew et al.;  Phys. Rev. B 85, 125428 (2012)]

        Returns
        -------

        Lm : float
            Layer modulus

        """
        Lm = 0.25 * (self.c2d[0][0] + self.c2d[1][1] + 2 * self.c2d[0][1])

        return Lm

    @property
    def layer_modulus(self):

        """
        Returns
        -------

        layer_modulus : float
           Layer modulus

        """
        return self.Lm

    @property
    def Y10(self):
        """

        2D Young's modulus or in-plane stiffness: Y[10] = [c11c22 - c12^2]/[c22]

        Returns
        -------

        Y10 : float
            2D Young's modulus (in-plane stiffness)

        """

        Y10 = (self.c2d[0][0] * self.c2d[1][1] - self.c2d[0][1] * self.c2d[0][1]) / (
            self.c2d[1][1]
        )

        return Y10

    @property
    def Y01(self):
        """

        2D Young's modulus or in-plane stiffness: Y[01] = [c11c22 - c12^2]/[c11]

        Returns
        -------
        Y01 : float
            2D Young's modulus (in-plane stiffness)

        """

        Y01 = (self.c2d[0][0] * self.c2d[1][1] - self.c2d[0][1] * self.c2d[0][1]) / (
            self.c2d[0][0]
        )

        return Y01

    @property
    def nu10(self):
        """

        2D Poisson's ratio;  nu10 = c12/c22

        Returns
        -------
        nu10 : float
            2D Poisson's ratio

        """

        nu10 = self.c2d[0][1] / self.c2d[1][1]

        return nu10

    @property
    def nu01(self):
        """

        2D Poisson's ratio; nu01 = c12/c11

        Returns
        -------
        nu01 : float
            2D Poisson's ratio

        """

        nu01 = self.c2d[0][1] / self.c2d[0][0]
        return nu01

    @property
    def G2d(self):
        """
        2D shear modulus; G2d = C66

        Returns
        -------
        G2d : float
            2D shear modulus

        """
        G2d = self.c2d[5][5]

        return G2d

    @property
    def shear_modulus_2d(self):
        """
        2D shear modulus; G2d = C66

        Returns
        -------
        G2d : float
            2D shear modulus

        """

        return self.G2d
    
    @property
    def elastic_stability(self):
        return stability.stability_test_2d(self.c2d, self.lattice_type)

    def print_properties(self):

        print("\n \n Elastic properties in two-dimensions \n")

        print(
            "[Useful refs. Andrew et al.;  Phys. Rev. B 85, 125428 (2012), Peng et al., Acta Mechanica 223 (2012), 2591-2596; Comput. Mater. Sci. 68, 320 (2013);  Mech. Mater. 64, 135 (2013)  ]"
        )
        print("                   ")
        print("-------------------------------------------------------")

        print("2D layer modulus (N/m)         :   %10.3f " % self.Lm)
        print("2D Young's modulus Y[10] (N/m) :   %10.3f " % self.Y10)
        print("2D Young's modulus Y[01] (N/m) :   %10.3f " % self.Y01)
        print("2D Shear modulus G (N/m)       :   %10.3f " % self.G2d)
        print("2D Poisson ratio v[10]         :   %10.3f " % self.nu10)
        print("2D Poisson ratio v[01]         :   %10.3f " % self.nu01)
        print("-------------------------------------------------------")
        
        #print(
        #    "Note:  The elastic stabilty test for 2D systems is not yet implemented. "
        #)

        #printer.print_warning_2D()



import numpy as np
import re
from itertools import groupby
from ..utils.constants import *
from ..utils.elements import ELEMENTS, elements_inversed

from ..comms import printer
from ..tests import symmetry

from ..core import ElasticProperties
from ..core import Structure


class AbinitOutput:

    """
    This class contains methods to parse the outputs from
    Abinit. It requires the scf/RF output and the ddb analysis
    output.
    """

    def __init__(self, infile="abinit.out", anaddbfile="abinit2.out"):

        self.infile = infile
        self.anaddbfile = anaddbfile

        self.elastic_tensor = None
        self.compaliance_tensor = None
        self.structure = None
        self.text = None
        self.density = None
        self._parse_output()

        # self.elastic_properties = ElasticProperties(
        #     self.elastic_tensor, self.structure)#, self.crystal_type)

        return

    def _parse_ddb(self):
        """
        This funcion reads the elastic tensor from the abinit output file
        after running anaddb.
        """

        rf = open(self.anaddbfile, "r")
        datamat = rf.read()
        rf.close()

        ET = re.findall(
            r"Elastic\s*Tensor\s*\(relaxed\s*ion\)\s*[\sA-Za-z:\d\^()]*\n([-\s0-9.]*)",
            datamat,
        )
        ET2 = np.array([float(x) for x in ET[0].split()])
        elastic_tensor = ET2.reshape(6, 6)

        return np.matrix(elastic_tensor)

    def _parse_output(self):
        """
        This function reads the abinit output for scf and RF calculations
        and initializes the class variables.

        """

        rf = open(self.infile)
        self.text = rf.read()
        rf.close()
        data = self.text
        mass = np.array(
            [float(x) for x in re.findall(r"amu\s*([E+=0-9.\s]*)\n", data)[0].split()]
        )
        volume = float(
            re.findall("Unit\s*cell\s*volume\s*ucvol\s*=\s*([-+0-9.E\s]*)", data)[-1]
        )
        # converting from Bohr^3 to Angstrom^3
        volume = volume * (0.529177249) ** 3

        nions = int(re.findall(r"\bnatom\b\s*([0-9]*)", data)[-1])

        # counting the number of atoms per type
        typat = [int(x) for x in re.findall(r"\btypat\b\s*([0-9\s]*)", data)[0].split()]
        species_grouped = [i[0] for i in groupby(typat)]
        iontype = [typat.count(i) for i in species_grouped]
        natoms = np.array([typat.count(i) for i in species_grouped])

        # Sometimes LATTYP is not present.
        # Only parse if available.
        lattice_type = re.findall("LATTYP.*", data)
        if lattice_type:
            lattyp = lattice_type[-1]

        full_latt = re.findall(r"\(Bohr,Bohr\^-1\):*\n([-+0-9.RG=\(\)\s\n]*)\n", data)[
            -1
        ].split()
        r1 = [float(x) for x in full_latt[1:4]]
        r2 = [float(x) for x in full_latt[9:12]]
        r3 = [float(x) for x in full_latt[17:20]]
        lattice = np.array((r1, r2, r3), dtype="float64")
        # converting from Bohr to Angstroms
        lattice = 0.529177249 * lattice

        positions = np.array(
            [
                float(x)
                for x in re.findall(r"\bxred\b\s*([0-9-+.\sE]*)\n", data)[-1].split()
            ]
        )
        positions = positions.reshape(nions, 3)

        self.lattice_constant = [
            0.529177249 * float(x)
            for x in re.findall(r"acell\d\s*([-+0-9.E\s]*)Bohr", data)[-1].split()
        ]

        # cell parameters
        A = float(self.lattice_constant[0])
        B = float(self.lattice_constant[1])
        C = float(self.lattice_constant[2])

        print(
            "Lattice parameters (in Angs.): a = %10.5f      b = %10.5f     c = %10.5f"
            % (A, B, C)
        )

        # external pressure in GPa
        self.pressure = float(re.findall(r"Pressure=\s*([0-9-+.E\s]*)\s*GPa", data)[-1])

        # atomic numbers
        znucl = [
            int(float(x)) for x in re.findall(r"znucl\s*([0-9.\s]*)\n", data)[0].split()
        ]

        species = [elements_inversed[x] for x in znucl]

        atomic_numbers = np.zeros(nions, dtype=np.int32)
        k = 0
        for i in range(len(iontype)):
            for j in range(iontype[i]):
                atomic_numbers[k] = ELEMENTS[species[i]]
                k = k + 1

        print("Atomic numbers")
        print(atomic_numbers)
        atomic_numbers = atomic_numbers
        symbols = [elements_inversed[x] for x in atomic_numbers]
        cell = (lattice, positions, atomic_numbers)

        print("Mass of atoms (in g/mol units): ")
        print((np.array(mass)))
        print("Number of atoms: %d" % np.sum(natoms))

        total_mass = 0.0
        for i in range(len(natoms)):
            total_mass = total_mass + natoms[i] * mass[i]
        print("Total mass (in g/mol): %10.4f " % total_mass)
        print("Volume of the cell (in Ang^3 units): %10.4f " % volume)

        # converting the units
        volume *= 1.0e-30  # from Angstrom to meters
        total_mass *= 1.0e-3  # from gram to kg
        density = total_mass / (volume * N_avogadro)
        self.density = density
        print("\nDensity (in kg/m^3 units ): %10.5f" % density)
        print("External Pressure (in GPa units ): %10.5f" % self.pressure)

        c = self._parse_ddb()

        print("\nPrinting Cij matrix as read from Abinit DDB output.\n")
        np.set_printoptions(precision=4, suppress=True)
        printer.printMatrix(c)

        self.elastic_tensor = c.copy()

        self.compaliance_tensor = self.elastic_tensor.I
        # print(self.elastic_tensor)
        print(
            "\n \n printing CNEW: Modified matrix in correct order (in GPa units)... \n For example- to generate input for the ELATE code [https://github.com/fxcoudert/elate] \n"
        )

        # in GPa
        self.elastic_tensor = self.elastic_tensor * 100

        np.set_printoptions(precision=3, suppress=True)

        printer.printMatrix(self.elastic_tensor)

        print(
            (
                "\n Checking if the modified matrix CNEW is symmetric: i.e. Cij = Cji:  %10s"
                % symmetry.check_symmetric(self.elastic_tensor)
            )
        )
        self.structure = Structure(symbols, positions, lattice)

        return



import numpy as np
from scipy.spatial import Voronoi
from ..core import Surface
import pyvista as pv
import trimesh

__author__ = "Pedram Tavadze"
__maintainer__ = "Pedram Tavadze"
__email__ = "petavazohi@mail.wvu.edu"
__date__ = "March 31, 2020"


class Lines:
    def __init__(self, verts=None, faces=None):

        self.verts = verts
        self.faces = faces

        self.pyvista_line = pv.PolyData()
        self.trimesh_line = None
        self.connectivity = []

        self._get_connectivity()

    @property
    def nface(self):
        return len(self.faces)

    def _get_connectivity(self):
        for iface in range(self.nface):
            self.connectivity.append(
                [self.faces[iface][0],
                 self.faces[iface][-1]])  # to connect the 1st and last point
            for ipoint in range(len(self.faces[iface]) - 1):
                point_1 = self.faces[ipoint]
                point_2 = self.faces[ipoint + 1]
                self.connectivity.append([point_1, point_2])

    def _create_pyvista(self):
        cell = []
        for iline in self.connectivity:
            cell.append([2, iline[0], iline[1]])
        self.pyvista_line.lines = cell

    def _create_trimesh(self):
        entries = []
        for iline in self.connectivity:
            entries.append(trimesh.path.entries.Line(iline))

            self.trimesh_line = trimesh.path.path.Path(entries=entries,
                                                       vertices=self.verts)


class BrillouinZone(Surface):
    """
    A Surface object with verts, faces and line representation, representing
    the BrillouinZone
    """
    def __init__(self, reciprocal_lattice, supercell):
        """
        Parameters
        ----------
        reciprocal_lattice : (3,3) float
            Reciprocal Lattice

        """

        self.reciprocal = reciprocal_lattice * max(supercell)
        # for ix in range(3):
        # self.reciprocal[:,ix]*=supercell[ix]
        verts, faces = self.wigner_seitz()

        Surface.__init__(self, verts=verts, faces=faces)

        self._fix_normals_direction()

        #self.pyvista_obj.face_normals*=-1
        # self.pyvista_obj['scalars'] = [0]*len(faces)
        # self.pyvista_obj.set_active_scalars('scalars')

        self.lines = Lines(verts, faces)

    def wigner_seitz(self):
        """
        
        Returns
        -------
        TYPE
            Using the Wigner-Seitz Method, this function finds the 1st 
            Brillouin Zone in terms of vertices and faces 
        """

        kpoints = []
        for i in range(-1, 2):
            for j in range(-1, 2):
                for k in range(-1, 2):
                    vec = i * self.reciprocal[0] + j * \
                        self.reciprocal[1] + k * self.reciprocal[2]
                    kpoints.append(vec)
        brill = Voronoi(np.array(kpoints))
        faces = []
        for idict in brill.ridge_dict:
            if idict[0] == 13 or idict[1] == 13:
                faces.append(brill.ridge_dict[idict])

        verts = brill.vertices
        return np.array(verts), np.array(faces)

    def _fix_normals_direction(self):
        # directions = np.zeros_like(self.centers)
        for iface in range(self.nfaces):
            center = self.centers[iface]
            n1 = center / np.linalg.norm(center)
            n2 = self.face_normals[iface]

            correction = np.sign(np.dot(n1, n2))
            self.face_normals[iface] = self.face_normals[iface] * correction
            self.pyvista_obj.face_normals[
                iface] = self.pyvista_obj.face_normals[iface] * correction
            # self.trimesh_obj.face_normals[iface]*=correction



from .elastic_properties import ElasticProperties
from .elastic_properties_2d import ElasticProperties2D
from .structure import Structure
from .surface import Surface
from .isosurface import Isosurface
from .brillouin_zone import BrillouinZone
from .elate import ELATE